@@ -121,7 +121,9 @@
 "XVkbd*menu.show_keypad.label: Show Keypad?",
 "XVkbd*menu.show_functionkey.label: Show Function Keys?",
 "XVkbd*menu.use_xtest.label: Use XTEST Extension?",
+"XVkbd*menu.quick_modifiers.label: Enable \"Quick Modifiers\"?",
 "XVkbd*menu.shift_lock.label: Lock Shift?",
+"XVkbd*menu.altgr_lock.label: Lock AltGr?",
 "XVkbd*menu.modifiers_lock.label: Lock Control, Alt and Meta?",
 "XVkbd*menu.open_display.label: Connect to Remote Display...",
 "XVkbd*menu.close_display.label: Revert to Local Display",
@@ -219,12 +221,13 @@
 "XVkbd*deadkey_panel*dead_doubleacute.label: \\264\\264",
 "XVkbd*deadkey_panel*dead_tilde.label: ~",
 "XVkbd*deadkey_panel*dead_abovedot.label: \\267",
-"XVkbd.customizations: default,belgian,danish,french,french2,\
+"XVkbd.customizations: default,belgian,danish,fitaly,french,french2,\
   german,greek,hebrew,icelandic,italian,jisx6004/ja,norwegian,\
   portuguese,slovene,spanish,swedish,swissgerman,turkish,uk,latin1",
 "XVkbd*layout_panel*default.label: Default",
 "XVkbd*layout_panel*belgian.label: Belgian",
 "XVkbd*layout_panel*danish.label: Danish",
+"XVkbd*layout_panel*fitaly.label: Fitaly",
 "XVkbd*layout_panel*french.label: French",
 "XVkbd*layout_panel*french2.label: French (2)",
 "XVkbd*layout_panel*german.label: German",
