@@ -28,9 +28,11 @@ struct appres_struct {
   Boolean xtest;
   Boolean jump_pointer;
   Boolean jump_pointer_always;
+  Boolean quick_modifiers;
   Boolean altgr_lock;
   Boolean shift_lock;
   Boolean modifiers_lock;
+  Boolean num_lock_state;
   Boolean auto_repeat;
   Boolean modal_keytop;
   int modal_threshold;
