@@ -1,6 +1,6 @@
 /*
  * xvkbd - Virtual Keyboard for X Window System
- * (Version 2.4, 2002-10-02)
+ * (Version 2.5a, 2003-06-25)
  *
  * Copyright (C) 2000-2002 by Tom Sato <VEF00200@nifty.ne.jp>
  * http://member.nifty.ne.jp/tsato/
@@ -52,7 +52,7 @@
 #endif
 
 #include "resources.h"
-#define PROGRAM_NAME_WITH_VERSION "xvkbd (v2.4)"
+#define PROGRAM_NAME_WITH_VERSION "xvkbd (v2.5a)"
 
 /*
  * Default keyboard layout is hardcoded here.
@@ -222,18 +222,27 @@ static XtResource application_resources[] = {
     Offset(geometry), XtRImmediate, "" },
   { "debug", "Debug", XtRBoolean, sizeof(Boolean),
      Offset(debug), XtRImmediate, (XtPointer)FALSE },
+#ifdef USE_XTEST
   { "xtest", "XTest", XtRBoolean, sizeof(Boolean),
      Offset(xtest), XtRImmediate, (XtPointer)TRUE },
+#else
+  { "xtest", "XTest", XtRBoolean, sizeof(Boolean),
+     Offset(xtest), XtRImmediate, (XtPointer)FALSE },
+#endif
   { "jumpPointer", "JumpPointer", XtRBoolean, sizeof(Boolean),
      Offset(jump_pointer), XtRImmediate, (XtPointer)TRUE },
   { "jumpPointerAlways", "JumpPointer", XtRBoolean, sizeof(Boolean),
      Offset(jump_pointer_always), XtRImmediate, (XtPointer)TRUE },
+  { "quickModifiers", "QuickModifiers", XtRBoolean, sizeof(Boolean),
+     Offset(quick_modifiers), XtRImmediate, (XtPointer)TRUE },
   { "altgrLock", "ModifiersLock", XtRBoolean, sizeof(Boolean),
-     Offset(altgr_lock), XtRImmediate, (XtPointer)TRUE },
+     Offset(altgr_lock), XtRImmediate, (XtPointer)FALSE },
   { "shiftLock", "ModifiersLock", XtRBoolean, sizeof(Boolean),
      Offset(shift_lock), XtRImmediate, (XtPointer)FALSE },
   { "modifiersLock", "ModifiersLock", XtRBoolean, sizeof(Boolean),
      Offset(modifiers_lock), XtRImmediate, (XtPointer)FALSE },
+  { "numLockState", "NumLockState", XtRBoolean, sizeof(Boolean),
+     Offset(num_lock_state), XtRImmediate, (XtPointer)TRUE },
   { "autoRepeat", "AutoRepeat", XtRBoolean, sizeof(Boolean),
      Offset(auto_repeat), XtRImmediate, (XtPointer)TRUE },
   { "modalKeytop", "ModalKeytop", XtRBoolean, sizeof(Boolean),
@@ -341,6 +350,7 @@ static XrmOptionDescRec options[] = {
   { "-xsendevent", ".xtest", XrmoptionNoArg, "False" },
   { "-no-jump-pointer", ".jumpPointer", XrmoptionNoArg, "False" },
 #endif
+  { "-quick", ".quickModifiers", XrmoptionNoArg, "True" },
   { "-text", ".text", XrmoptionSepArg, NULL },
   { "-file", ".file", XrmoptionSepArg, NULL },
   { "-window", ".window", XrmoptionSepArg, NULL },
@@ -394,8 +404,6 @@ static int altgr_mask = 0;
 static int shift_state = 0;
 static int mouse_shift = 0;
 
-static Boolean num_lock_state = FALSE;
-
 static Display *target_dpy = NULL;
 
 static Window toplevel_parent = None;
@@ -1333,9 +1341,9 @@ static void RefreshShiftState(Boolean force)
   if (changed & altgr_mask) {
     Highlight("Mode_switch", cur_shift & altgr_mask);
   }
-  if (last_num_lock_state != num_lock_state) {
-    Highlight("Num_Lock", num_lock_state);
-    Highlight("keypad_panel*Num_Lock", num_lock_state);
+  if (last_num_lock_state != appres.num_lock_state) {
+    Highlight("Num_Lock", appres.num_lock_state);
+    Highlight("keypad_panel*Num_Lock", appres.num_lock_state);
   }
   if (last_target_dpy != target_dpy || last_focus != focused_window) {
     Highlight("Focus", focused_window != 0);
@@ -1382,7 +1390,7 @@ static void RefreshShiftState(Boolean force)
 
   last_shift_state = shift_state;
   last_mouse_shift = mouse_shift;
-  last_num_lock_state = num_lock_state;
+  last_num_lock_state = appres.num_lock_state;
   first = FALSE;
 }
 
@@ -1392,8 +1400,12 @@ static void RefreshShiftState(Boolean force)
  * which will be called as callback for the Command widgets, and we
  * only need to check which mouse button is pressed here.
  */
+static unsigned int n_key_repeat;
+
 static void ButtonDownAction(Widget w, XEvent *event, String *pars, Cardinal *n_pars)
 {
+  n_key_repeat = 0;
+
   switch (event->xbutton.button) {
   case Button2:
     mouse_shift |= ControlMask;
@@ -1412,6 +1424,9 @@ static void ButtonDownAction(Widget w, XEvent *event, String *pars, Cardinal *n_
  */
 static void ButtonUpAction(Widget w, XEvent *event, String *pars, Cardinal *n_pars)
 {
+  if (appres.quick_modifiers) {
+    if (n_key_repeat == 1) XtCallCallbacks(w, XtNcallback, NULL);
+  }
   mouse_shift = 0;
   RefreshShiftState(FALSE);
 }
@@ -1543,6 +1558,8 @@ static Widget display_status = None;
 
 #define DISPLAY_NAME_LENGTH 50
 
+static void SaveFunctionKey(void);  /* forward */
+
 static void OpenRemoteDisplay(Widget w, char *display_name, char *data)
 {
   static char name[DISPLAY_NAME_LENGTH + 10];
@@ -1640,11 +1657,21 @@ static void MenuSelected(Widget w, char *key)
   } else if (strcmp(key, "use_xtest") == 0) {
     appres.xtest = !appres.xtest;
     RefreshMainMenu();
+  } else if (strcmp(key, "quick_modifiers") == 0) {
+    appres.quick_modifiers = !appres.quick_modifiers;
+    SaveFunctionKey();
+    RefreshMainMenu();
   } else if (strcmp(key, "shift_lock") == 0) {
     appres.shift_lock = !appres.shift_lock;
+    SaveFunctionKey();
+    RefreshMainMenu();
+  } else if (strcmp(key, "altgr_lock") == 0) {
+    appres.altgr_lock = !appres.altgr_lock;
+    SaveFunctionKey();
     RefreshMainMenu();
   } else if (strcmp(key, "modifiers_lock") == 0) {
     appres.modifiers_lock = !appres.modifiers_lock;
+    SaveFunctionKey();
     RefreshMainMenu();
   } else if (strcmp(key, "open_display") == 0) {
     if (display_panel == None) {
@@ -1745,7 +1772,7 @@ static void KeyPressed(Widget w, char *key, char *data)
     if (shift_state & altgr_mask) SendKeyPressedEvent(NoSymbol, shift_state);
     shift_state ^= altgr_mask;
   } else if (strcmp(key, "Num_Lock") == 0) {
-    num_lock_state = !num_lock_state;
+    appres.num_lock_state = !appres.num_lock_state;
   } else if (strcmp(key, "Focus") == 0) {
     cur_shift = shift_state | mouse_shift;
     if (cur_shift & ShiftMask) {
@@ -1755,12 +1782,36 @@ static void KeyPressed(Widget w, char *key, char *data)
       GetFocusedWindow();
     }
   } else {
+    if (appres.quick_modifiers && mouse_shift == 0 && w != None) {
+      Window junk_w;
+      int junk_i;
+      unsigned junk_u;
+      int cur_x, cur_y;
+      Dimension btn_wd, btn_ht;
+
+      n_key_repeat = n_key_repeat + 1;
+      if (n_key_repeat == 1) return;
+
+      XtVaGetValues(w, XtNwidth, &btn_wd, XtNheight, &btn_ht, NULL);
+      XQueryPointer(dpy, XtWindow(w), &junk_w, &junk_w,
+		    &junk_i, &junk_i, &cur_x, &cur_y, &junk_u);
+
+      mouse_shift = 0;
+      if (cur_x < 0 && btn_ht < cur_y) {
+	mouse_shift |= alt_mask;  /* left-down */
+      } else {
+	if (cur_y < 0) mouse_shift |= ShiftMask;  /* up */
+	else if (btn_ht < cur_y) mouse_shift |= meta_mask;  /* down */
+	if (cur_x < 0) mouse_shift |= ControlMask;  /* left */
+	else if (btn_wd < cur_x) mouse_shift |= altgr_mask;  /* right */
+      }
+    }
     cur_shift = shift_state | mouse_shift;
     shifted = (cur_shift & ShiftMask);
     key1 = key;
     if (w != None) {
       if (sscanf(key, "pad%d,%d", &row, &col) == 2) {
-	if (num_lock_state) shifted = !shifted;
+	if (appres.num_lock_state) shifted = !shifted;
 	key1 = shifted ? keypad_shift[row][col]: keypad[row][col];
       } else {
 	found = FALSE;
@@ -2114,8 +2165,12 @@ static void RefreshMainMenu(void)
   XtVaSetValues(XtNameToWidget(main_menu, "*use_xtest"),
 		XtNrightBitmap, appres.xtest ? check_pixmap : None, NULL);
 #endif
+  XtVaSetValues(XtNameToWidget(main_menu, "*quick_modifiers"),
+		XtNrightBitmap, appres.quick_modifiers ? check_pixmap : None, NULL);
   XtVaSetValues(XtNameToWidget(main_menu, "*shift_lock"),
 		XtNrightBitmap, appres.shift_lock ? check_pixmap : None, NULL);
+  XtVaSetValues(XtNameToWidget(main_menu, "*altgr_lock"),
+		XtNrightBitmap, appres.altgr_lock ? check_pixmap : None, NULL);
   XtVaSetValues(XtNameToWidget(main_menu, "*modifiers_lock"),
 		XtNrightBitmap, appres.modifiers_lock ? check_pixmap : None, NULL);
 
@@ -2134,7 +2189,7 @@ static void MakeKeyboard(Boolean remake)
 #ifdef USE_XTEST
     "use_xtest",
 #endif
-    "shift_lock", "modifiers_lock", "",
+    "quick_modifiers", "shift_lock", "altgr_lock", "modifiers_lock", "",
     "open_display", "close_display", "",
     "quit" };
 
@@ -2347,9 +2402,10 @@ static struct fkey_struct {
 static void ReadFuncionKeys(void)
 {
   FILE *fp;
-  char str[200];
+  char str[200], key[200];
   struct fkey_struct *sp, *new_node;
   char len;
+  int val;
 
   if (appres.key_file[0] != '/' && getenv("HOME") != NULL)
     sprintf(fkey_filename, "%s/%s", getenv("HOME"), appres.key_file);
@@ -2360,7 +2416,17 @@ static void ReadFuncionKeys(void)
   if (fp == NULL) return;
 
   while (fgets(str, sizeof(str) - 1, fp)) {
-    if (isalpha(str[0])) {
+    if (str[0] == '#') {
+      sscanf(&str[1], "%s %d", key, &val);
+      if (strcmp(key, "quick_modifiers") == 0)
+	appres.quick_modifiers = val;
+      else if (strcmp(key, "shift_lock") == 0)
+	appres.shift_lock = val;
+      else if (strcmp(key, "altgr_lock") == 0)
+	appres.altgr_lock = val;
+      else if (strcmp(key, "modifiers_lock") == 0)
+	appres.modifiers_lock = val;
+    } else if (isalpha(str[0])) {
       len = strlen(str);
       if (str[len - 1] == '\n') str[len - 1] = '\0';
 
@@ -2476,7 +2542,7 @@ static void SaveFunctionKey(void)
   struct fkey_struct *sp;
   FILE *fp;
 
-  FKeyMenuSelected(None, NULL);
+  if (edit_fkey_panel != None) FKeyMenuSelected(None, NULL);
 
   fp = fopen(fkey_filename, "w");
   if (fp == NULL) {
@@ -2484,12 +2550,16 @@ static void SaveFunctionKey(void)
 	    PROGRAM_NAME, fkey_filename, strerror(errno));
     return;
   }
+  fprintf(fp, "#quick_modifiers %d\n", appres.quick_modifiers);
+  fprintf(fp, "#shift_lock %d\n", appres.shift_lock);
+  fprintf(fp, "#altgr_lock %d\n", appres.altgr_lock);
+  fprintf(fp, "#modifiers_lock %d\n", appres.modifiers_lock);
   for (sp = fkey_list; sp != NULL; sp = sp->next) {
     fprintf(fp, "%s\n", sp->value);
   }
   fclose(fp);
 
-  CloseFunctionKeyPanel();
+  if (edit_fkey_panel != None) CloseFunctionKeyPanel();
 }
 
 static void PopupFunctionKeyEditor(void)
@@ -2742,6 +2812,8 @@ int main(int argc, char *argv[])
       SendFileContent(appres.file);
     exit(0);
  } else {
+    ReadFuncionKeys();
+
     if (0 < strlen(appres.keys_normal))
       RedefineKeys(keys_normal, appres.keys_normal);
     if (0 < strlen(appres.keys_shift))
@@ -2775,6 +2847,8 @@ int main(int argc, char *argv[])
 	XtSetSensitive(XtNameToWidget(main_menu, "*completion"), FALSE);
       if (strlen(appres.customizations) == 0)
 	XtSetSensitive(XtNameToWidget(main_menu, "*select_layout"), FALSE);
+      if (XtNameToWidget(toplevel, "*Mode_switch") == None)
+	XtSetSensitive(XtNameToWidget(main_menu, "*altgr_lock"), FALSE);
     }
 
 #ifdef USE_XTEST
@@ -2807,8 +2881,6 @@ int main(int argc, char *argv[])
 #endif
     }
 
-    ReadFuncionKeys();
-
     SetIconBitmap(toplevel);
 
     XtAppMainLoop(app_con);
